--------------------------------------------------
-- Simple XON/XOFF Emulation
--
-- Requires firmware 2.10+
-- 
-- 2008-07-18: Initial release
-- 2009-02-23: Updated to fix obscure memory issue
--------------------------------------------------
function emulation.defaults(e)
 ---- USER PARAMETERS ----
-- e._message  = "IPBUFFER"          -- Optional sign-on message
-- e._password = "SMDR"               -- Optional password
 e._waitforxon = false               -- Wait for an XON before sending data?
 e._maximumdata = 2 * 1024 * 1024   -- 2Mb max in one transfer

 e._modem=false
 e._timeout=30000       -- Default command timeout for LAN
 if (e.peer=="127.0.0.1") -- Definitely shouldn't escape
 then
  e._modem=true
  e._timeout=15*60*1000 -- Large timeout for modem mode
 end
end


function emulation.init(e)
 e:defaults()
 e._version="2008-07-18"
 e._bytes=0
 e._stoppedsoon=false
end

function emulation.writeln(e,s)
 s=s or ""
 return e:write(s.."\r\n")
end

function emulation.wsf(e,f,...)
 f = f or ""
 return e:write(string.format(f.."\r\n",...))
end

function emulation.key(e,t)
 return e:read(1, t or 30000)
end

function emulation.keyline(e,echo,timeout)
 local s,t,l
 l=""
 while true
 do
  s,t=e:key(timeout or e._timeout or 30000)
  if t then return NULL end
  if ((echo==NULL) and e._echo) or echo
  then
   if (s~="\x02") and (s~="\n") then e:write(s) end
   if s=="\r" then e:write("\n") end
  end
  if s=="\x08"
  then
   if ((echo==NULL) and e._echo) or echo
   then
    if l=="" then e:write(" ") else e:write(" \x08") end
   end
   l=string.sub(l,1,-2)
  elseif s=="\n"
  then
  elseif s=="\x1b"
  then return NULL
  elseif s=="\r"
  then
   local cmd=string.match(l,"\x02([%a%d]*),*")  -- Control B
   if cmd then cmd=tonumber(cmd); l=string.sub(l, 2) end
   return l,cmd
  else
   l=l..s
  end
 end
end

function emulation.signon(e)
 local pw=""
 if e._message and (e._message~="") then e:writeln(e._message) end
 if not e._password or (e._password=="") then return true end
 while true
 do
  pw=e:keyline(false)
  if not pw then e:writeln("\r\nPassword timeout.") return false end
  if pw==e._password then return true end
  e:writeln("ERROR")
  e:writeln(e._message)
 end
end


function emulation.senddata(e)
 local pause=e._waitforxon or false
 local k,t
 e:adv(0)
 while true
 do
  while true
  do
   k=e:read(1)
   if not k or (k=="") then break end
   if k=="\x13" then pause=true end
   if k=="\x11" then pause=false end
  end
  while pause
  do
   k,t=e:key()
   if t then return false end
   if k=="\x11" then pause=false end
  end
  local s,n=e:get(250)
  if n<=0 then return true end
  e:adv(n)
  if not e:write(s) then return false end
  e._bytes=e._bytes+n
  if e._maximumdata and (e._bytes>=e._maximumdata)
  then
   e:stopsoon()
   e._stoppedsoon=true
  end
 end
end


function emulation.connect(e)
 e:init()
 if e._modem then e:sleep(2) end
 if not e:signon() then return end
 e:open(e.n)
 if not e:senddata() then return end
 if e._stoppedsoon then e:writeln("\r\nMORE DATA TO FOLLOW") end
 e:writeln("\r\nEND DATA\r\n")
 e:erase()
 e:sleep(15)
 e:close()
end

